import pytest
from pydantic_forms.exceptions import FormValidationError

from gso.services.partners import get_partner_by_id
from test.workflows import assert_complete, run_workflow


@pytest.mark.workflow()
def test_modify_partner_success(partner_factory):
    partner = partner_factory(
        name="new_name",
        email="myemail@gmail.com",
    )
    result, _, _ = run_workflow(
        "task_modify_partners",
        [
            {"partners": partner["partner_id"]},
            {
                "name": "GEANT-TEST-CREATION",
                "email": "goat-test-creation@geant.org",
            },
        ],
    )
    assert_complete(result)

    partner_db = get_partner_by_id(partner_id=partner["partner_id"])
    assert partner_db.name == "GEANT-TEST-CREATION"
    assert partner_db.email == "goat-test-creation@geant.org"


@pytest.mark.workflow()
def test_modify_partner_with_same_date_success(partner_factory):
    partner = partner_factory(
        name="new_name",
        email="myemail@gmail.com",
    )
    result, _, _ = run_workflow(
        "task_modify_partners",
        [
            {"partners": partner["partner_id"]},
            {
                "name": "new_name",
                "email": "myemail@gmail.com",
            },
        ],
    )
    assert_complete(result)

    partner_db = get_partner_by_id(partner_id=partner["partner_id"])
    assert partner_db.name == "new_name"
    assert partner_db.email == "myemail@gmail.com"


@pytest.mark.workflow()
def test_modify_partner_with_duplicate_name_or_email_fails(partner_factory):
    partner_factory(
        name="new_name",
        email="myemail@gmail.com",
    )
    partner_2 = partner_factory(
        name="new_name_2",
        email="myemail2@gmail.com",
    )

    with pytest.raises(FormValidationError) as error:
        run_workflow(
            "task_modify_partners",
            [
                {"partners": partner_2["partner_id"]},
                {
                    "name": "new_name",
                    "email": "myemail@gmail.com",
                },
            ],
        )

    assert error.value.errors[0]["msg"] == "Partner with this name already exists."
    assert error.value.errors[1]["msg"] == "Partner with this email already exists."


@pytest.mark.workflow()
def test_modify_partner_with_invalid_input_fails(partner_factory):
    partner = partner_factory(
        name="new_name_2",
        email="myemail2@gmail.com",
    )
    with pytest.raises(FormValidationError) as error:
        run_workflow(
            "task_modify_partners",
            [
                {"partners": partner["partner_id"]},
                {
                    "name": "Kenneth Boyle",
                    "email": "invalid_email",
                },
            ],
        )

    errors = error.value.errors

    email_error = errors[0]
    assert email_error["loc"] == ("email",)
    assert "valid email address" in email_error["msg"]
